@@ -9,6 +9,7 @@ from typing import ClassVar
 from typing import Dict
 from typing import Generic
 from typing import List
+from typing import NewType
 from typing import Optional
 from typing import Set
 from typing import Type
@@ -16,6 +17,9 @@ from typing import TypeVar
 from typing import Union
 import uuid
 
+from typing_extensions import get_args as get_args
+from typing_extensions import Literal as Literal
+
 from sqlalchemy import BIGINT
 from sqlalchemy import BigInteger
 from sqlalchemy import Column
@@ -63,6 +67,7 @@ from sqlalchemy.testing import is_
 from sqlalchemy.testing import is_false
 from sqlalchemy.testing import is_not
 from sqlalchemy.testing import is_true
+from sqlalchemy.testing import Variation
 from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.util import compat
 from sqlalchemy.util.typing import Annotated
@@ -448,6 +453,31 @@ class MappedColumnTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         else:
             args = ()
 
+        # anno only: global anno_str, anno_str_optional, anno_str_mc
+        # anno only: global anno_str_optional_mc, anno_str_mc_nullable
+        # anno only: global anno_str_optional_mc_notnull
+        # anno only: global newtype_str
+
+        anno_str = Annotated[str, 50]
+        anno_str_optional = Annotated[Optional[str], 30]
+
+        newtype_str = NewType("MyType", str)
+
+        anno_str_mc = Annotated[str, mapped_column()]
+        anno_str_optional_mc = Annotated[Optional[str], mapped_column()]
+        anno_str_mc_nullable = Annotated[str, mapped_column(nullable=True)]
+        anno_str_optional_mc_notnull = Annotated[
+            Optional[str], mapped_column(nullable=False)
+        ]
+
+        decl_base.registry.update_type_annotation_map(
+            {
+                anno_str: String(50),
+                anno_str_optional: String(30),
+                newtype_str: String(40),
+            }
+        )
+
         class User(decl_base):
             __tablename__ = "users"
 
@@ -464,6 +494,41 @@ class MappedColumnTest(fixtures.TestBase, testing.AssertsCompiledSQL):
                 *args, nullable=True
             )
 
+            # test #9177 cases
+            anno_1a: Mapped[anno_str] = mapped_column(*args)
+            anno_1b: Mapped[anno_str] = mapped_column(*args, nullable=True)
+
+            anno_2a: Mapped[anno_str_optional] = mapped_column(*args)
+            anno_2b: Mapped[anno_str_optional] = mapped_column(
+                *args, nullable=False
+            )
+
+            anno_3a: Mapped[anno_str_mc] = mapped_column(*args)
+            anno_3b: Mapped[anno_str_mc] = mapped_column(*args, nullable=True)
+            anno_3c: Mapped[Optional[anno_str_mc]] = mapped_column(*args)
+
+            anno_4a: Mapped[anno_str_optional_mc] = mapped_column(*args)
+            anno_4b: Mapped[anno_str_optional_mc] = mapped_column(
+                *args, nullable=False
+            )
+
+            anno_5a: Mapped[anno_str_mc_nullable] = mapped_column(*args)
+            anno_5b: Mapped[anno_str_mc_nullable] = mapped_column(
+                *args, nullable=False
+            )
+
+            anno_6a: Mapped[anno_str_optional_mc_notnull] = mapped_column(
+                *args
+            )
+            anno_6b: Mapped[anno_str_optional_mc_notnull] = mapped_column(
+                *args, nullable=True
+            )
+
+            newtype_1a: Mapped[newtype_str] = mapped_column(*args)
+            newtype_1b: Mapped[newtype_str] = mapped_column(
+                *args, nullable=True
+            )
+
         is_false(User.__table__.c.lnnl_rndf.nullable)
         is_false(User.__table__.c.lnnl_rnnl.nullable)
         is_true(User.__table__.c.lnnl_rnl.nullable)
@@ -472,6 +537,20 @@ class MappedColumnTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         is_false(User.__table__.c.lnl_rnnl.nullable)
         is_true(User.__table__.c.lnl_rnl.nullable)
 
+        is_false(User.__table__.c.anno_1a.nullable)
+        is_true(User.__table__.c.anno_1b.nullable)
+        is_true(User.__table__.c.anno_2a.nullable)
+        is_false(User.__table__.c.anno_2b.nullable)
+        is_false(User.__table__.c.anno_3a.nullable)
+        is_true(User.__table__.c.anno_3b.nullable)
+        is_true(User.__table__.c.anno_3c.nullable)
+        is_true(User.__table__.c.anno_4a.nullable)
+        is_false(User.__table__.c.anno_4b.nullable)
+        is_true(User.__table__.c.anno_5a.nullable)
+        is_false(User.__table__.c.anno_5b.nullable)
+        is_false(User.__table__.c.anno_6a.nullable)
+        is_true(User.__table__.c.anno_6b.nullable)
+
         # test #8410
         is_false(User.__table__.c.lnnl_rndf._copy().nullable)
         is_false(User.__table__.c.lnnl_rnnl._copy().nullable)
@@ -519,6 +598,41 @@ class MappedColumnTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         is_true(MyClass.__table__.c.data_two.nullable)
         eq_(MyClass.__table__.c.data_three.type.length, 50)
 
+    def test_pep484_newtypes_as_typemap_keys(
+        self, decl_base: Type[DeclarativeBase]
+    ):
+
+        # anno only: global str50, str30, str3050
+
+        str50 = NewType("str50", str)
+        str30 = NewType("str30", str)
+        str3050 = NewType("str30", str50)
+
+        decl_base.registry.update_type_annotation_map(
+            {str50: String(50), str30: String(30), str3050: String(150)}
+        )
+
+        class MyClass(decl_base):
+            __tablename__ = "my_table"
+
+            id: Mapped[str50] = mapped_column(primary_key=True)
+            data_one: Mapped[str30]
+            data_two: Mapped[str50]
+            data_three: Mapped[Optional[str30]]
+            data_four: Mapped[str3050]
+
+        eq_(MyClass.__table__.c.data_one.type.length, 30)
+        is_false(MyClass.__table__.c.data_one.nullable)
+
+        eq_(MyClass.__table__.c.data_two.type.length, 50)
+        is_false(MyClass.__table__.c.data_two.nullable)
+
+        eq_(MyClass.__table__.c.data_three.type.length, 30)
+        is_true(MyClass.__table__.c.data_three.nullable)
+
+        eq_(MyClass.__table__.c.data_four.type.length, 150)
+        is_false(MyClass.__table__.c.data_four.nullable)
+
     def test_extract_base_type_from_pep593(
         self, decl_base: Type[DeclarativeBase]
     ):
@@ -1188,83 +1302,6 @@ class MappedColumnTest(fixtures.TestBase, testing.AssertsCompiledSQL):
                 id: Mapped[int] = mapped_column(primary_key=True)
                 data: Mapped["fake"]  # noqa
 
-    @testing.variation("use_callable", [True, False])
-    @testing.variation("include_generic", [True, False])
-    def test_enum_explicit(self, use_callable, include_generic):
-        # anno only: global FooEnum
-
-        class FooEnum(enum.Enum):
-            foo = enum.auto()
-            bar = enum.auto()
-
-        if use_callable:
-            tam = {FooEnum: Enum(FooEnum, length=500)}
-        else:
-            tam = {FooEnum: Enum(FooEnum, length=500)}
-        if include_generic:
-            tam[enum.Enum] = Enum(enum.Enum)
-        Base = declarative_base(type_annotation_map=tam)
-
-        class MyClass(Base):
-            __tablename__ = "mytable"
-
-            id: Mapped[int] = mapped_column(primary_key=True)
-            data: Mapped[FooEnum]
-
-        is_true(isinstance(MyClass.__table__.c.data.type, Enum))
-        eq_(MyClass.__table__.c.data.type.length, 500)
-        is_(MyClass.__table__.c.data.type.enum_class, FooEnum)
-
-    def test_enum_generic(self):
-        """test for #8859"""
-        # anno only: global FooEnum
-
-        class FooEnum(enum.Enum):
-            foo = enum.auto()
-            bar = enum.auto()
-
-        Base = declarative_base(
-            type_annotation_map={enum.Enum: Enum(enum.Enum, length=42)}
-        )
-
-        class MyClass(Base):
-            __tablename__ = "mytable"
-
-            id: Mapped[int] = mapped_column(primary_key=True)
-            data: Mapped[FooEnum]
-
-        is_true(isinstance(MyClass.__table__.c.data.type, Enum))
-        eq_(MyClass.__table__.c.data.type.length, 42)
-        is_(MyClass.__table__.c.data.type.enum_class, FooEnum)
-
-    def test_enum_default(self, decl_base):
-        """test #8859.
-
-        We now have Enum in the default SQL lookup map, in conjunction with
-        a mechanism that will adapt it for a given enum type.
-
-        This relies on a search through __mro__ for the given type,
-        which in other tests we ensure does not actually function if
-        we aren't dealing with Enum (or some other type that allows for
-        __mro__ lookup)
-
-        """
-        # anno only: global FooEnum
-
-        class FooEnum(enum.Enum):
-            foo = "foo"
-            bar_value = "bar"
-
-        class MyClass(decl_base):
-            __tablename__ = "mytable"
-
-            id: Mapped[int] = mapped_column(primary_key=True)
-            data: Mapped[FooEnum]
-
-        is_true(isinstance(MyClass.__table__.c.data.type, Enum))
-        eq_(MyClass.__table__.c.data.type.length, 9)
-        is_(MyClass.__table__.c.data.type.enum_class, FooEnum)
-
     def test_type_dont_mis_resolve_on_superclass(self):
         """test for #8859.
 
@@ -1326,7 +1363,9 @@ class MappedColumnTest(fixtures.TestBase, testing.AssertsCompiledSQL):
 
     def test_type_secondary_resolution(self):
         class MyString(String):
-            def _resolve_for_python_type(self, python_type, matched_type):
+            def _resolve_for_python_type(
+                self, python_type, matched_type, matched_on_flattened
+            ):
                 return String(length=42)
 
         Base = declarative_base(type_annotation_map={str: MyString})
@@ -1341,6 +1380,264 @@ class MappedColumnTest(fixtures.TestBase, testing.AssertsCompiledSQL):
         eq_(MyClass.__table__.c.data.type.length, 42)
 
 
+class EnumOrLiteralTypeMapTest(fixtures.TestBase, testing.AssertsCompiledSQL):
+    __dialect__ = "default"
+
+    @testing.variation("use_callable", [True, False])
+    @testing.variation("include_generic", [True, False])
+    @testing.variation("set_native_enum", ["none", True, False])
+    def test_enum_explicit(
+        self, use_callable, include_generic, set_native_enum: Variation
+    ):
+        # anno only: global FooEnum
+
+        class FooEnum(enum.Enum):
+            foo = enum.auto()
+            bar = enum.auto()
+
+        kw = {"length": 500}
+
+        if set_native_enum.none:
+            expected_native_enum = True
+        elif set_native_enum.set_native_enum:
+            kw["native_enum"] = True
+            expected_native_enum = True
+        elif set_native_enum.not_set_native_enum:
+            kw["native_enum"] = False
+            expected_native_enum = False
+        else:
+            set_native_enum.fail()
+
+        if use_callable:
+            tam = {FooEnum: Enum(FooEnum, **kw)}
+        else:
+            tam = {FooEnum: Enum(FooEnum, **kw)}
+
+        if include_generic:
+            tam[enum.Enum] = Enum(enum.Enum)
+        Base = declarative_base(type_annotation_map=tam)
+
+        class MyClass(Base):
+            __tablename__ = "mytable"
+
+            id: Mapped[int] = mapped_column(primary_key=True)
+            data: Mapped[FooEnum]
+
+        is_true(isinstance(MyClass.__table__.c.data.type, Enum))
+        eq_(MyClass.__table__.c.data.type.length, 500)
+        is_(MyClass.__table__.c.data.type.enum_class, FooEnum)
+        is_(MyClass.__table__.c.data.type.native_enum, expected_native_enum)
+
+    @testing.variation("set_native_enum", ["none", True, False])
+    def test_enum_generic(self, set_native_enum: Variation):
+        """test for #8859"""
+        # anno only: global FooEnum
+
+        class FooEnum(enum.Enum):
+            foo = enum.auto()
+            bar = enum.auto()
+
+        kw = {"length": 42}
+
+        if set_native_enum.none:
+            expected_native_enum = True
+        elif set_native_enum.set_native_enum:
+            kw["native_enum"] = True
+            expected_native_enum = True
+        elif set_native_enum.not_set_native_enum:
+            kw["native_enum"] = False
+            expected_native_enum = False
+        else:
+            set_native_enum.fail()
+
+        Base = declarative_base(
+            type_annotation_map={enum.Enum: Enum(enum.Enum, **kw)}
+        )
+
+        class MyClass(Base):
+            __tablename__ = "mytable"
+
+            id: Mapped[int] = mapped_column(primary_key=True)
+            data: Mapped[FooEnum]
+
+        is_true(isinstance(MyClass.__table__.c.data.type, Enum))
+        eq_(MyClass.__table__.c.data.type.length, 42)
+        is_(MyClass.__table__.c.data.type.enum_class, FooEnum)
+        is_(MyClass.__table__.c.data.type.native_enum, expected_native_enum)
+
+    def test_enum_default(self, decl_base):
+        """test #8859.
+
+        We now have Enum in the default SQL lookup map, in conjunction with
+        a mechanism that will adapt it for a given enum type.
+
+        This relies on a search through __mro__ for the given type,
+        which in other tests we ensure does not actually function if
+        we aren't dealing with Enum (or some other type that allows for
+        __mro__ lookup)
+
+        """
+        # anno only: global FooEnum
+
+        class FooEnum(enum.Enum):
+            foo = "foo"
+            bar_value = "bar"
+
+        class MyClass(decl_base):
+            __tablename__ = "mytable"
+
+            id: Mapped[int] = mapped_column(primary_key=True)
+            data: Mapped[FooEnum]
+
+        is_true(isinstance(MyClass.__table__.c.data.type, Enum))
+        eq_(MyClass.__table__.c.data.type.length, 9)
+        is_(MyClass.__table__.c.data.type.enum_class, FooEnum)
+
+    @testing.variation(
+        "sqltype", ["custom", "base_enum", "specific_enum", "string"]
+    )
+    @testing.variation("indicate_type_explicitly", [True, False])
+    def test_pep586_literal(
+        self, decl_base, sqltype: Variation, indicate_type_explicitly
+    ):
+        """test #9187."""
+
+        # anno only: global Status
+
+        Status = Literal["to-do", "in-progress", "done"]
+
+        if sqltype.custom:
+
+            class LiteralSqlType(types.TypeDecorator):
+                impl = types.String
+                cache_ok = True
+
+                def __init__(self, literal_type: Any) -> None:
+                    super().__init__()
+                    self._possible_values = get_args(literal_type)
+
+            our_type = mapped_col_type = LiteralSqlType(Status)
+        elif sqltype.specific_enum:
+            our_type = mapped_col_type = Enum(
+                "to-do", "in-progress", "done", native_enum=False
+            )
+        elif sqltype.base_enum:
+            our_type = Enum(enum.Enum, native_enum=False)
+            mapped_col_type = Enum(
+                "to-do", "in-progress", "done", native_enum=False
+            )
+        elif sqltype.string:
+            our_type = mapped_col_type = String(50)
+        else:
+            sqltype.fail()
+
+        decl_base.registry.update_type_annotation_map({Status: our_type})
+
+        class Foo(decl_base):
+            __tablename__ = "footable"
+
+            id: Mapped[int] = mapped_column(primary_key=True)
+
+            if indicate_type_explicitly:
+                status: Mapped[Status] = mapped_column(mapped_col_type)
+            else:
+                status: Mapped[Status]
+
+        is_true(isinstance(Foo.__table__.c.status.type, type(our_type)))
+
+        if sqltype.custom:
+            eq_(
+                Foo.__table__.c.status.type._possible_values,
+                ("to-do", "in-progress", "done"),
+            )
+        elif sqltype.specific_enum or sqltype.base_enum:
+            eq_(
+                Foo.__table__.c.status.type.enums,
+                ["to-do", "in-progress", "done"],
+            )
+            is_(Foo.__table__.c.status.type.native_enum, False)
+
+    @testing.variation("indicate_type_explicitly", [True, False])
+    def test_pep586_literal_defaults_to_enum(
+        self, decl_base, indicate_type_explicitly
+    ):
+        """test #9187."""
+
+        # anno only: global Status
+
+        Status = Literal["to-do", "in-progress", "done"]
+
+        if indicate_type_explicitly:
+            expected_native_enum = True
+        else:
+            expected_native_enum = False
+
+        class Foo(decl_base):
+            __tablename__ = "footable"
+
+            id: Mapped[int] = mapped_column(primary_key=True)
+
+            if indicate_type_explicitly:
+                status: Mapped[Status] = mapped_column(
+                    Enum("to-do", "in-progress", "done")
+                )
+            else:
+                status: Mapped[Status]
+
+        is_true(isinstance(Foo.__table__.c.status.type, Enum))
+
+        eq_(
+            Foo.__table__.c.status.type.enums,
+            ["to-do", "in-progress", "done"],
+        )
+        is_(Foo.__table__.c.status.type.native_enum, expected_native_enum)
+
+    @testing.variation("override_in_type_map", [True, False])
+    @testing.variation("indicate_type_explicitly", [True, False])
+    def test_pep586_literal_checks_the_arguments(
+        self, decl_base, indicate_type_explicitly, override_in_type_map
+    ):
+        """test #9187."""
+
+        # anno only: global NotReallyStrings
+
+        NotReallyStrings = Literal["str1", 17, False]
+
+        if override_in_type_map:
+            decl_base.registry.update_type_annotation_map(
+                {NotReallyStrings: JSON}
+            )
+
+        if not override_in_type_map and not indicate_type_explicitly:
+            with expect_raises_message(
+                ArgumentError,
+                "Can't create string-based Enum datatype from non-string "
+                "values: 17, False.  Please provide an explicit Enum "
+                "datatype for this Python type",
+            ):
+
+                class Foo(decl_base):
+                    __tablename__ = "footable"
+
+                    id: Mapped[int] = mapped_column(primary_key=True)
+                    status: Mapped[NotReallyStrings]
+
+        else:
+            # if we override the type in the type_map or mapped_column,
+            # then we can again use a Literal with non-strings
+            class Foo(decl_base):
+                __tablename__ = "footable"
+
+                id: Mapped[int] = mapped_column(primary_key=True)
+
+                if indicate_type_explicitly:
+                    status: Mapped[NotReallyStrings] = mapped_column(JSON)
+                else:
+                    status: Mapped[NotReallyStrings]
+
+            is_true(isinstance(Foo.__table__.c.status.type, JSON))
+
+
 class MixinTest(fixtures.TestBase, testing.AssertsCompiledSQL):
     __dialect__ = "default"
 
@@ -2501,21 +2798,43 @@ class BackendTests(fixtures.TestBase):
 
     @testing.variation("native_enum", [True, False])
     @testing.variation("include_column", [True, False])
+    @testing.variation("python_type", ["enum", "literal"])
     def test_schema_type_actually_works(
-        self, connection, decl_base, include_column, native_enum
+        self,
+        connection,
+        decl_base,
+        include_column,
+        native_enum,
+        python_type: Variation,
     ):
         """test that schema type bindings are set up correctly"""
 
         # anno only: global Status
 
-        class Status(enum.Enum):
-            PENDING = "pending"
-            RECEIVED = "received"
-            COMPLETED = "completed"
+        if python_type.enum:
+
+            class Status(enum.Enum):
+                PENDING = "pending"
+                RECEIVED = "received"
+                COMPLETED = "completed"
+
+            enum_argument = [Status]
+            test_value = Status.RECEIVED
+        elif python_type.literal:
+            Status = Literal[  # type: ignore
+                "pending", "received", "completed"
+            ]
+            enum_argument = ["pending", "received", "completed"]
+            test_value = "received"
+        else:
+            python_type.fail()
 
         if not include_column and not native_enum:
             decl_base.registry.update_type_annotation_map(
-                {enum.Enum: Enum(enum.Enum, native_enum=False)}
+                {
+                    enum.Enum: Enum(enum.Enum, native_enum=False),
+                    Literal: Enum(enum.Enum, native_enum=False),
+                }
             )
 
         class SomeClass(decl_base):
@@ -2525,7 +2844,11 @@ class BackendTests(fixtures.TestBase):
 
             if include_column:
                 status: Mapped[Status] = mapped_column(
-                    Enum(Status, native_enum=bool(native_enum))
+                    Enum(
+                        *enum_argument,
+                        native_enum=bool(native_enum),
+                        name="status",
+                    )
                 )
             else:
                 status: Mapped[Status]
@@ -2533,12 +2856,12 @@ class BackendTests(fixtures.TestBase):
         decl_base.metadata.create_all(connection)
 
         with Session(connection) as sess:
-            sess.add(SomeClass(id=1, status=Status.RECEIVED))
+            sess.add(SomeClass(id=1, status=test_value))
             sess.commit()
 
             eq_(
                 sess.scalars(
                     select(SomeClass.status).where(SomeClass.id == 1)
                 ).first(),
-                Status.RECEIVED,
+                test_value,
             )
