@@ -1,3 +1,5 @@
+from unittest.mock import Mock
+
 import sqlalchemy as sa
 from sqlalchemy import delete
 from sqlalchemy import ForeignKey
@@ -6,13 +8,13 @@ from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy import select
+from sqlalchemy import String
 from sqlalchemy import table
 from sqlalchemy import testing
 from sqlalchemy import true
 from sqlalchemy import update
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm import backref
-from sqlalchemy.orm import mapper
 from sqlalchemy.orm import relationship
 from sqlalchemy.orm import Session
 from sqlalchemy.orm import sessionmaker
@@ -24,7 +26,6 @@ from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_
 from sqlalchemy.testing import mock
 from sqlalchemy.testing.fixtures import fixture_session
-from sqlalchemy.testing.mock import Mock
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from test.orm import _fixtures
@@ -46,8 +47,8 @@ class BindIntegrationTest(_fixtures.FixtureTest):
         users_unbound = users.to_metadata(m2)
         addresses_unbound = addresses.to_metadata(m2)
 
-        mapper(Address, addresses_unbound)
-        mapper(
+        self.mapper_registry.map_imperatively(Address, addresses_unbound)
+        self.mapper_registry.map_imperatively(
             User,
             users_unbound,
             properties={
@@ -103,8 +104,8 @@ class BindIntegrationTest(_fixtures.FixtureTest):
         users_unbound = users.to_metadata(m2)
         addresses_unbound = addresses.to_metadata(m2)
 
-        mapper(Address, addresses_unbound)
-        mapper(
+        self.mapper_registry.map_imperatively(Address, addresses_unbound)
+        self.mapper_registry.map_imperatively(
             User,
             users_unbound,
             properties={
@@ -155,7 +156,7 @@ class BindIntegrationTest(_fixtures.FixtureTest):
     def test_bind_from_metadata(self):
         users, User = self.tables.users, self.classes.User
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
 
         session = fixture_session()
 
@@ -186,6 +187,10 @@ class BindIntegrationTest(_fixtures.FixtureTest):
         (lambda Address: {"mapper": Address}, "e2"),
         (lambda Address: {"clause": Query([Address])._statement_20()}, "e2"),
         (lambda addresses: {"clause": select(addresses)}, "e2"),
+        (lambda Dingaling: {"mapper": Dingaling}, "e4"),
+        (lambda addresses_view: {"clause": addresses_view}, "e4"),
+        (lambda addresses_view: {"clause": select(addresses_view)}, "e4"),
+        (lambda users_view: {"clause": select(users_view)}, "e2"),
         (
             lambda User, addresses: {
                 "mapper": User,
@@ -260,21 +265,48 @@ class BindIntegrationTest(_fixtures.FixtureTest):
             self.tables.addresses,
             self.classes.User,
         )
+        Dingaling = self.classes.Dingaling
 
-        mapper(User, users, properties={"addresses": relationship(Address)})
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(
+            User, users, properties={"addresses": relationship(Address)}
+        )
+        self.mapper_registry.map_imperatively(Address, addresses)
+
+        users_view = table("users", Column("id", Integer, primary_key=True))
+        addresses_view = table(
+            "addresses",
+            Column("id", Integer, primary_key=True),
+            Column("user_id", Integer),
+            Column("email_address", String),
+        )
+        j = users_view.join(
+            addresses_view, users_view.c.id == addresses_view.c.user_id
+        )
+        self.mapper_registry.map_imperatively(
+            Dingaling,
+            j,
+            properties={
+                "user_t_id": users_view.c.id,
+                "address_id": addresses_view.c.id,
+            },
+        )
 
         e1 = engines.testing_engine()
         e2 = engines.testing_engine()
         e3 = engines.testing_engine()
+        e4 = engines.testing_engine()
 
         testcase = testing.resolve_lambda(
             testcase,
             User=User,
             Address=Address,
+            Dingaling=Dingaling,
             e1=e1,
             e2=e2,
             e3=e3,
+            e4=e4,
+            users_view=users_view,
+            addresses_view=addresses_view,
             addresses=addresses,
             users=users,
         )
@@ -282,16 +314,40 @@ class BindIntegrationTest(_fixtures.FixtureTest):
         sess = Session(e3)
         sess.bind_mapper(User, e1)
         sess.bind_mapper(Address, e2)
+        sess.bind_mapper(Dingaling, e4)
+        sess.bind_table(users_view, e2)
 
-        engine = {"e1": e1, "e2": e2, "e3": e3}[expected]
-        conn = sess.connection(**testcase)
+        engine = {"e1": e1, "e2": e2, "e3": e3, "e4": e4}[expected]
+        conn = sess.connection(bind_arguments=testcase)
         is_(conn.engine, engine)
 
         sess.close()
 
+    @testing.combinations(True, False)
+    def test_dont_mutate_binds(self, empty_dict):
+        users, User = (
+            self.tables.users,
+            self.classes.User,
+        )
+
+        mp = self.mapper_registry.map_imperatively(User, users)
+
+        sess = fixture_session()
+
+        if empty_dict:
+            bind_arguments = {}
+        else:
+            bind_arguments = {"mapper": mp}
+        sess.execute(select(1), bind_arguments=bind_arguments)
+
+        if empty_dict:
+            eq_(bind_arguments, {})
+        else:
+            eq_(bind_arguments, {"mapper": mp})
+
     @testing.combinations(
         (
-            lambda session, Address: session.query(Address),
+            lambda session, Address: session.query(Address).statement,
             lambda Address: {"mapper": inspect(Address), "clause": mock.ANY},
             "e2",
         ),
@@ -322,6 +378,22 @@ class BindIntegrationTest(_fixtures.FixtureTest):
             lambda User: {"clause": mock.ANY, "mapper": inspect(User)},
             "e1",
         ),
+        (
+            lambda User: update(User)
+            .execution_options(synchronize_session=False)
+            .values(name="not ed")
+            .where(User.name == "ed"),
+            lambda User: {"clause": mock.ANY, "mapper": inspect(User)},
+            "e1",
+        ),
+        (
+            lambda User: insert(User).values(name="not ed"),
+            lambda User: {
+                "clause": mock.ANY,
+                "mapper": inspect(User),
+            },
+            "e1",
+        ),
     )
     def test_bind_through_execute(
         self, statement, expected_get_bind_args, expected_engine_name
@@ -333,8 +405,10 @@ class BindIntegrationTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        mapper(User, users, properties={"addresses": relationship(Address)})
-        mapper(Address, addresses)
+        self.mapper_registry.map_imperatively(
+            User, users, properties={"addresses": relationship(Address)}
+        )
+        self.mapper_registry.map_imperatively(Address, addresses)
 
         e1 = engines.testing_engine()
         e2 = engines.testing_engine()
@@ -351,7 +425,7 @@ class BindIntegrationTest(_fixtures.FixtureTest):
                 canary.get_bind(**kw)
                 return Session.get_bind(self, **kw)
 
-        sess = GetBindSession(e3, future=True)
+        sess = GetBindSession(e3)
         sess.bind_mapper(User, e1)
         sess.bind_mapper(Address, e2)
 
@@ -373,7 +447,15 @@ class BindIntegrationTest(_fixtures.FixtureTest):
         engine = {"e1": e1, "e2": e2, "e3": e3}[expected_engine_name]
 
         with mock.patch(
-            "sqlalchemy.orm.context.ORMCompileState.orm_setup_cursor_result"
+            "sqlalchemy.orm.context." "ORMCompileState.orm_setup_cursor_result"
+        ), mock.patch(
+            "sqlalchemy.orm.context.ORMCompileState.orm_execute_statement"
+        ), mock.patch(
+            "sqlalchemy.orm.bulk_persistence."
+            "BulkORMInsert.orm_execute_statement"
+        ), mock.patch(
+            "sqlalchemy.orm.bulk_persistence."
+            "BulkUDCompileState.orm_setup_cursor_result"
         ):
             sess.execute(statement)
 
@@ -397,7 +479,9 @@ class BindIntegrationTest(_fixtures.FixtureTest):
             testing.db,
         )
 
-        mapper(self.classes.User, self.tables.users)
+        self.mapper_registry.map_imperatively(
+            self.classes.User, self.tables.users
+        )
         u_object = self.classes.User()
 
         assert_raises_message(
@@ -412,11 +496,11 @@ class BindIntegrationTest(_fixtures.FixtureTest):
     def test_bound_connection(self):
         users, User = self.tables.users, self.classes.User
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
         c = testing.db.connect()
         sess = Session(bind=c)
         sess.begin()
-        transaction = sess._legacy_transaction()
+        transaction = sess.get_transaction()
         u = User(name="u1")
         sess.add(u)
         sess.flush()
@@ -440,46 +524,53 @@ class BindIntegrationTest(_fixtures.FixtureTest):
     def test_bound_connection_transactional(self):
         User, users = self.classes.User, self.tables.users
 
-        mapper(User, users)
+        self.mapper_registry.map_imperatively(User, users)
         with testing.db.connect() as c:
 
-            sess = Session(bind=c, autocommit=False)
+            sess = Session(bind=c)
+
             u = User(name="u1")
             sess.add(u)
             sess.flush()
+
+            # new in 2.0:
+            # autobegin occurred, so c is in a transaction.
+
+            assert c.in_transaction()
             sess.close()
+
+            # .close() does a rollback, so that will end the
+            # transaction on the connection.  This is how it was
+            # working before also even if transaction was started.
+            # is this what we really want?
             assert not c.in_transaction()
+
             assert (
                 c.exec_driver_sql("select count(1) from users").scalar() == 0
             )
 
-            sess = Session(bind=c, autocommit=False)
+            sess = Session(bind=c)
             u = User(name="u2")
             sess.add(u)
             sess.flush()
             sess.commit()
-            assert not c.in_transaction()
-            assert (
-                c.exec_driver_sql("select count(1) from users").scalar() == 1
-            )
 
-            with c.begin():
-                c.exec_driver_sql("delete from users")
-            assert (
-                c.exec_driver_sql("select count(1) from users").scalar() == 0
-            )
+            # new in 2.0:
+            # commit OTOH doesn't actually do a commit.
+            # so still in transaction due to autobegin
+            assert c.in_transaction()
 
-        with testing.db.connect() as c:
-            trans = c.begin()
-            sess = Session(bind=c, autocommit=True)
+            sess = Session(bind=c)
             u = User(name="u3")
             sess.add(u)
             sess.flush()
-            assert c.in_transaction()
-            trans.commit()
+            sess.rollback()
+
+            # like .close(), rollback() also ends the transaction
             assert not c.in_transaction()
+
             assert (
-                c.exec_driver_sql("select count(1) from users").scalar() == 1
+                c.exec_driver_sql("select count(1) from users").scalar() == 0
             )
 
 
@@ -507,8 +598,7 @@ class SessionBindTest(fixtures.MappedTest):
         meta = MetaData()
         test_table.to_metadata(meta)
 
-        assert meta.tables["test_table"].bind is None
-        mapper(Foo, meta.tables["test_table"])
+        cls.mapper_registry.map_imperatively(Foo, meta.tables["test_table"])
 
     def test_session_bind(self):
         Foo = self.classes.Foo
@@ -522,7 +612,7 @@ class SessionBindTest(fixtures.MappedTest):
                 f = Foo()
                 sess.add(f)
                 sess.flush()
-                assert sess.query(Foo).get(f.id) is f
+                assert sess.get(Foo, f.id) is f
             finally:
                 if hasattr(bind, "close"):
                     bind.close()
@@ -580,14 +670,18 @@ class GetBindTest(fixtures.MappedTest):
 
     @classmethod
     def setup_mappers(cls):
-        mapper(cls.classes.ClassWMixin, cls.tables.w_mixin_table)
-        mapper(cls.classes.BaseClass, cls.tables.base_table)
-        mapper(
+        cls.mapper_registry.map_imperatively(
+            cls.classes.ClassWMixin, cls.tables.w_mixin_table
+        )
+        cls.mapper_registry.map_imperatively(
+            cls.classes.BaseClass, cls.tables.base_table
+        )
+        cls.mapper_registry.map_imperatively(
             cls.classes.JoinedSubClass,
             cls.tables.joined_sub_table,
             inherits=cls.classes.BaseClass,
         )
-        mapper(
+        cls.mapper_registry.map_imperatively(
             cls.classes.ConcreteSubClass,
             cls.tables.concrete_sub_table,
             inherits=cls.classes.BaseClass,
@@ -629,7 +723,7 @@ class GetBindTest(fixtures.MappedTest):
         session = self._fixture({})
         assert_raises_message(
             sa.exc.UnboundExecutionError,
-            "Could not locate a bind configured on mapper mapped class",
+            "Could not locate a bind configured on mapper ",
             session.get_bind,
             self.classes.BaseClass,
         )
@@ -640,7 +734,7 @@ class GetBindTest(fixtures.MappedTest):
 
         assert_raises_message(
             sa.exc.UnboundExecutionError,
-            "Could not locate a bind configured on mapper mapped class",
+            "Could not locate a bind configured on mapper ",
             session.get_bind,
             self.classes.ConcreteSubClass,
         )
