@@ -807,7 +807,7 @@ class LazyTest(_fixtures.FixtureTest):
         """
 
         @registry.mapped
-        class A(object):
+        class A:
             __tablename__ = "a"
 
             id = Column(Integer, primary_key=True)
@@ -816,7 +816,7 @@ class LazyTest(_fixtures.FixtureTest):
             b = relationship("B")
 
         @registry.mapped
-        class B(object):
+        class B:
             __tablename__ = "b"
 
             id = Column(Integer, primary_key=True)
@@ -955,7 +955,7 @@ class LazyTest(_fixtures.FixtureTest):
             properties={"addresses": relationship(Address, backref="user")},
         )
         self.mapper_registry.map_imperatively(Address, addresses)
-        sess = fixture_session(autoflush=False, future=True)
+        sess = fixture_session(autoflush=False)
         ad = sess.query(Address).filter_by(id=1).one()
         assert ad.user.id == 7
 
@@ -1583,7 +1583,7 @@ class TypeCoerceTest(fixtures.MappedTest, testing.AssertsExecutionResults):
             return sa.cast(col, Integer)
 
         def bind_expression(self, col):
-            return sa.cast(col, String(50))
+            return sa.cast(sa.type_coerce(col, Integer), String(50))
 
     @classmethod
     def define_tables(cls, metadata):
